#include <windows.h>

#include "conf.hpp"
#include "gfx.hpp"
#include "global.hpp"

void draw_grid(HDC hdc)
{
    // Draw a black frame
    {
        HBRUSH hbr=(HBRUSH)SelectObject(hdc, GetStockObject(WHITE_BRUSH));
        Rectangle(hdc, 0, 0, conf::playground_width,
            conf::playground_height);
        SelectObject(hdc, hbr);
    }

    // Draw the grid (points)
    for(int x=0; x<conf::playground_width; x+=conf::playground_unit_width)
    for(int y=0; y<conf::playground_height; y+=conf::playground_unit_height)
        SetPixel(hdc, x, y, 0);

}

namespace {
void draw_button(HDC hdc, COLORREF color, int x, int y)
{
    int const center_x=(x+1)*conf::playground_unit_width;
    int const center_y=(y+1)*conf::playground_unit_height;
    COLORREF const white=RGB(0xff, 0xff, 0xff);

    {
        RECT re;
        re.left=center_x-2; re.top=center_y-2;
        re.right=center_x+3; re.bottom=center_y+3;
        FillRect(hdc, &re, (HBRUSH)GetStockObject(BLACK_BRUSH));
    }

    SetPixel(hdc, center_x-2, center_y-2, white);
    SetPixel(hdc, center_x+2, center_y-2, white);
    SetPixel(hdc, center_x-2, center_y+2, white);
    SetPixel(hdc, center_x+2, center_y+2, white);

    {
        HBRUSH hbr=CreateSolidBrush(color);
        RECT re; re.left=center_x-1; re.top=center_y-1;
            re.right=center_x+2; re.bottom=center_y+2;
        FillRect(hdc, &re, hbr);
        DeleteObject(hbr);
    }


}

}// anonymous namespace
void draw_button(HDC hdc, button::type t, int x, int y)
{
    switch(t){
        case button::white:
            draw_button(hdc, RGB(0xff, 0xff, 0xff), x, y);
            break;
        case button::gray:
            draw_button(hdc, RGB(0xaa, 0xaa, 0xaa), x, y);
            break;
        case button::white_pressed:
            draw_button(hdc, RGB(0x33, 0x33, 0x33), x, y);
            break;
    }
}

void draw_robot(HDC hdc, direction::type dir, int x, int y)
{
    char const* res;
    switch(dir){
        case direction::up: res="HERBERT_UP"; break;
        case direction::down: res="HERBERT_DOWN"; break;
        case direction::right: res="HERBERT_RIGHT"; break;
        case direction::left: res="HERBERT_LEFT"; break;
        default: res="HERBERT_UP";
    }
    HDC sdc=CreateCompatibleDC(hdc);
    HBITMAP bitmap=LoadBitmap(global::hinstance, res);

    BITMAP bm;
    GetObject(bitmap, sizeof(bm), &bm);

    SelectObject(sdc, LoadBitmap(global::hinstance, res));

    BitBlt(hdc, (x+1)*conf::playground_unit_width-5, (y+1)*conf::playground_unit_height-5,
        bm.bmWidth, bm.bmHeight, sdc, 0, 0, SRCCOPY);

    DeleteDC(sdc);
}

void draw_wall(HDC hdc, int x, int y)
{
    int const center_x=conf::playground_unit_width*(x+1);
    int const center_y=conf::playground_unit_height*(y+1);

    int const wall_size=2;

    RECT re;
    re.left=center_x-wall_size; re.right=center_x+wall_size+1;
    re.top=center_y-wall_size; re.bottom=center_y+wall_size+1;


    FillRect(hdc, &re, (HBRUSH)GetStockObject(BLACK_BRUSH));
}

void draw_trail(HDC hdc, int x, int y, direction::type t)
{
    int const from_x=conf::playground_unit_width*(x+1);
    int const from_y=conf::playground_unit_height*(y+1);

    int to_x=from_x, to_y=from_y;

    switch(t){
        case direction::up:
            to_y=conf::playground_unit_height*(y+1-1)-1;
            break;
        case direction::down:
            to_y=conf::playground_unit_height*(y+1+1)+1;
            break;
        case direction::left:
            to_x=conf::playground_unit_width*(x+1-1)-1;
            break;
        case direction::right:
            to_x=conf::playground_unit_width*(x+1+1)+1;
            break;
    };


    MoveToEx(hdc, from_x, from_y, NULL);
    LineTo(hdc, to_x, to_y);

}
